// Template ID, Device Name and Auth Token are provided by the Blynk.Cloud
// See the Device Info tab, or Template settings
#define BLYNK_TEMPLATE_ID           "[your template id]"
#define BLYNK_DEVICE_NAME           "[your device name]"
#define BLYNK_AUTH_TOKEN            "[your auth token]"

#include <Proximity.h>
#include <Adafruit_NeoPixel.h>
#include "IIRFilter.h"
#include "Adafruit_CPlay_Speaker.h"
#define BLYNK_PRINT SerialUSB
#define LED_PIN 8
#define LED_COUNT 10
boolean using_sin = true;
Proximity prox;
Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);

#include <ESP8266_Lib.h>
#include <BlynkSimpleShieldEsp8266.h>

char auth[] = BLYNK_AUTH_TOKEN;

// Your WiFi credentials.
// Set password to "" for open networks.
char ssid[] = "[your network name]";
char pass[] = "[your network password]";

// Hardware Serial on Mega, Leonardo, Micro...
#define EspSerial Serial1

// or Software Serial on Uno, Nano...
//#include <SoftwareSerial.h>
//SoftwareSerial EspSerial(2, 3); // RX, TX

#include <Adafruit_CircuitPlayground.h>

// Your ESP8266 baud rate:
#define ESP8266_BAUD 115200

ESP8266 wifi(&EspSerial);

void setup()
{
  // Debug console
  SerialUSB.begin(115200);
  CircuitPlayground.begin();
  prox.begin(100);
  // Set ESP8266 baud rate
  EspSerial.begin(ESP8266_BAUD);
  delay(10);
  
  strip.begin();
  Blynk.begin(auth, wifi, ssid, pass);
  // You can also specify server:
  //Blynk.begin(auth, wifi, ssid, pass, "blynk.cloud", 80);
  //Blynk.begin(auth, wifi, ssid, pass, IPAddress(192,168,1,100), 8080);
}

// variables declaration
//status report var
bool isAvailable = false;
String availableStatus = "Available";
String unavailableStatus = "Unavailable";
String availablePrefix = "SLACK_AVAILABLE:";
String unavailablePrefix = "SLACK_UNAVAILABLE:";

// take a walk vars
#define  THRESHOLD           1.1     
#define  SAMPLE_RATE_HZ      50.0    
#define  STEP_FLASH_MS       500 
#define FILTER_A       { 1.0, -2.08554010137, 1.54484341043, -0.394448994245 }
#define FILTER_B       { 0.00810678935123, 0.0243203680537, 0.0243203680537, 0.00810678935123 }
const uint32_t periodMS = int(1.0/SAMPLE_RATE_HZ*1000.0);     // Period of time (in milliseconds) between accelerometer samples.
IIRFilter<4,4> filter((float[])FILTER_A, (float[])FILTER_B);  // The IIR filter that will use the coefficients defined above.
uint32_t lastMS = 0;           // Time of last loop iteration.
uint32_t lastSampleMS = 0;     // Time of last accelerometer sample.
int pixelClearMS = 0;      // Amount of time left before clearing the NeoPixels and turning them off.
int stepsTH = 0;
uint32_t lastSoundMS = 0;
const uint32_t soundPeriodMS = 2000;
long red = 0xFF0000;
long green = 0x00FF00;
long blue = 0x0000FF;
long purple = 0x9100FF;
long yellow = 0xFFEF00;
long brown = 0x973E03;
long white = 0x000000;
int ledIndex = 0;
long colors[] = {red, green, blue, purple, yellow, brown};
int colorIndex = 0;

// meeting alert vars
#define MY_PROTOCOL NECX
#define MY_BITS 32
#define POWER 0x1818D02F
#define MUTE 0x181838C8
bool tvTurnOff = false;
bool tvMute = false;

// hours report vars
int light;
int LIGHTS_ON = 800;
int LIGHTS_OFF = 700;
int workDay = false;

// take a walk alert
BLYNK_WRITE(V1)
{
  int pinValue = param.asInt();
  if(pinValue) {
    setAllLedColor(white);
    takeAWalk();
    if(isAvailable) {
      setAllLedColor(green);  
    } else {
      setAllLedColor(red);
    }
  } 
}

// take a walk function - measures the steps taken while playing sound and changing the led colors
// stops when the required amount of steps was taken
void takeAWalk() {
  lastMS = millis();
  stepsTH = 0;
  while(stepsTH < 400) {
    // Compute how much time ellapsed since the last loop iteration.
    uint32_t currentMS = millis();
    uint32_t deltaMS = currentMS - lastMS;
    if(currentMS >= (lastSoundMS + soundPeriodMS)) {
      CircuitPlayground.playTone(500, 100);
      lastSoundMS = currentMS;
      CircuitPlayground.setPixelColor((ledIndex - 1) % 10, white);
      CircuitPlayground.setPixelColor(ledIndex % 10, colors[colorIndex % 6]);
      ledIndex++; 
      colorIndex++;
    }
    // Check if enough time has elapsed to take a new accelerometer reading.
    if (currentMS >= (lastSampleMS + periodMS)) {
      // Grab a new accelerometer reading and compute the magnitude of the
      // acceleration vector defined by the X, Y, Z axis.  This is just computing:
      // sqrt(x^2 + y^2 + z^2)
      CircuitPlayground.lis.read();
      float mag = sqrt(pow(CircuitPlayground.lis.x_g, 2.0) +
                       pow(CircuitPlayground.lis.y_g, 2.0) +
                       pow(CircuitPlayground.lis.z_g, 2.0));
  
      // Apply the low pass filter to the magnitude reading.
      float filtered = filter.filter(mag);
  
      // Turn on the NeoPixels red if the filtered magnitude exceeds the threshold.
      if (filtered > THRESHOLD) {
        stepsTH++;
      }
      
      // Update time of last sample.
      lastSampleMS = currentMS;
    }
    // Update time of last frame.
    lastMS = currentMS;
    Serial.println(stepsTH);
  }
}

// Meeting alert - focus time, turn tv off/ mute according to user selection
BLYNK_WRITE(V2)
{
  int pinValue = param.asInt();
  if(pinValue) {
    if(tvTurnOff) {
      CircuitPlayground.irSend.send(MY_PROTOCOL,POWER,MY_BITS);
    }
    if(tvMute){
      CircuitPlayground.irSend.send(MY_PROTOCOL,MUTE,MY_BITS);
    }
  } 
}

// user set tv off
BLYNK_WRITE(V4) {
  int pinValue = param.asInt();
  tvTurnOff = pinValue;
}

// user set tv mute
BLYNK_WRITE(V5) {
  int pinValue = param.asInt();
  tvTurnOff = pinValue;
}

// check if the day has started or ended
void checkDayStartEnd() {
  light = CircuitPlayground.lightSensor();
  if(light > LIGHTS_ON && !workDay) {
    Blynk.virtualWrite(V0, "DAY_START");
    workDay = true;
    Serial.println("DAY START");
  } else if(light < LIGHTS_OFF && workDay) {
    Blynk.virtualWrite(V0, "DAY_END");
    workDay = false;
    Serial.println("DAY END");
  }
}

// user set available status
BLYNK_WRITE(V6) {
  String pinValue = param.asString();
  availableStatus = pinValue;
  Blynk.virtualWrite(V7, unavailableStatus);
}

// user set unavailable status
BLYNK_WRITE(V7) {
  String pinValue = param.asString();
  unavailableStatus = pinValue;
  Blynk.virtualWrite(V7, unavailableStatus);
}

// checks if the users is available/unavaile to report status change
void checkStatusReport() {
  if (prox.close() && !isAvailable) {
    Serial.println("User proximity is high, changing status to available");
    isAvailable = true;    
    Blynk.virtualWrite(V0, availablePrefix + availableStatus);
    setAllLedColor(green);
  } else if(!prox.close() && isAvailable) {
    Serial.println("User proximity is low, changing status to unavailable");
    isAvailable = false;    
    Blynk.virtualWrite(V0, unavailablePrefix + unavailableStatus);
    setAllLedColor(red);
  }
}

// helper function
void setAllLedColor(long color) {
  for(int i = 0; i < 10; i++){
    CircuitPlayground.setPixelColor(i, color);
  }
}

// user set proximity threshold 
BLYNK_WRITE(V3)
{
  int pinValue = param.asInt();
  prox.setThreshold(pinValue);
  Serial.print("Updating proximity to: ");
  Serial.println(pinValue);
}

void loop()
{
  Blynk.run();
  checkDayStartEnd();
  if(workDay) {
    checkStatusReport();
  }
}
